/*-------------------------------------------------------------------
  fu.c - Fossil utils

  9-26-97 add FOSSset_hook function.
  1-05-96 FOSSrts setting bit 1(dtr) change to set/reset bit 2(rts)
 11-09-95 - setdtr/setrts backwards, changed around
|---------------------------------------------------------------------*/
#ifdef M9
#define ASM _asm
#else          // BORLAND31, default
#define ASM asm
#endif

// values returned from get status(func003) in AX register                                      
#define ST_OUTBUF_EMPTY         0x4000
#define ST_OUTBUF_NOT_FULL      0x2000
#define ST_INPUT_BUFF_OVERRUN   0x0200
#define ST_INPUT_HAS_CHARS      0x0100
#define ST_CD_SIGNAL            0x0080

typedef struct {
  WORD  strsiz;
  BYTE  majver;
  BYTE  minver;
  char far *ident;
  WORD  ibufr;  // input buf size
  WORD  ifree;  // num bytes in input queue
  WORD  obufr;  // output buf size
  WORD  ofree;  // num butes in output queue
  BYTE  swidth;
  BYTE  sheight;
  BYTE  baud;
} FossInfo;

void FOSSbreak(int port, int setflag);
void FOSSloopback(int port, int setflag);
void FOSSrts(int port, int setflag);
void FOSSdtr(int port, int setflag);
void FOSSclose(int port);
unsigned int FOSSinit(int port);
void FOSSputc_0B(int port, unsigned char c);
void FOSSputc(int, unsigned char);
int FOSSgetc(int);
unsigned int FOSSstatus(int);
unsigned int FOSSbaud(int port, unsigned char baud);
unsigned int FOSSbaudExt(int port, WORD baud, WORD byte_size,
                                WORD parity, WORD stop_bits);
void FOSSflowCtrl(int, unsigned char);
unsigned int FOSSpeek(int);
unsigned int FOSSctlck(int, unsigned char);
unsigned int FOSSreadblk(int port, int count, unsigned char *buffer);
unsigned int FOSSwriteblk(int port, int count, unsigned char *buffer);
unsigned int FOSSgetinfo(int port, FossInfo *fi);
void FOSSgetcursor(int *x, int *y);
void FOSSsetcursor(int x, int y);
void FOSSscrchar(char c);

// following are values for standard int14 bios get status function(3)
#define LNSTAT_DATARDY    0x0100
#define LNSTAT_OVRRUN     0x0200
#define LNSTAT_PARITY     0x0400
#define LNSTAT_FRAMING    0x0800
#define LNSTAT_BREAK      0x1000
#define LNSTAT_THRE       0x2000
#define LNSTAT_TE         0x4000
#define LNSTAT_TIMEOUT    0x8000

#define MDMSTAT_DCTS       0x0001
#define MDMSTAT_DDSR       0x0002
#define MDMSTAT_DRI        0x0004
#define MDMSTAT_DCD        0x0008
#define MDMSTAT_CTS        0x0010
#define MDMSTAT_DSR        0x0020
#define MDMSTAT_RI         0x0040
#define MDMSTAT_CD         0x0080

/*----------------------------------------------------------------
| BIOSsendchar -
|-----------------------------------------------------------------*/
void BIOSsendchar(int port, unsigned char c)
{
 WORD RegDX= port;
 WORD RegAX= 0x0100;  // AH = func 1, send char

  RegAX |= c;  // AL = char
  ASM {
    mov DX, RegDX
    mov AX, RegAX
    int 0x14
  }
}

/*----------------------------------------------------------------
| BIOSgetchar - return lobyte(AL) = char, hibyte(AH) = line status
|-----------------------------------------------------------------*/
unsigned int BIOSgetchar(int port)
{
 WORD RegDX= port;
 WORD RegAX= 0x0200;  // AH = func 2, receive char

  ASM {
    mov DX, RegDX
    mov AX, RegAX
    int 0x14
    mov RegAX, AX
  }
  return RegAX;
}

/*----------------------------------------------------------------
| BIOSstatus - func03.  Returns hibyte(ah)=line status, lobyte=modemstatus
|-----------------------------------------------------------------*/
unsigned int BIOSstatus(int port)
{
 WORD RegDX= port;
 WORD RegAX= 0x0300;  // AH = func 3, bios port status

  ASM {
    mov DX, RegDX
    mov AX, RegAX
    int 0x14
    mov RegAX, AX
  }
  return RegAX;
}

/*----------------------------------------------------------------
| BIOSconfig - Uses BIOS 0 or 4 config function.  Uses 0 if possible,
|  otherwise uses function 4.
|-----------------------------------------------------------------*/
unsigned int BIOSconfig(int port, DWORD baud, WORD byte_size,
                                WORD parity, WORD stop_bits)
{
 WORD RegDX;
 WORD RegCX;
 WORD RegBX;
 WORD RegAX;
 WORD bios0=7;  // default to 9600 baud for BIOS 0 config
 WORD bval=7;  // default to 9600
 WORD bsize;

  if (baud <= 38400) // use bios0 config call
  {
    switch (baud)
    {
      case 110:   bios0=0; break;
      case 150:   bios0=1; break;
      case 300:   bios0=2; break;
      case 600:   bios0=3; break;
      case 1200:  bios0=4; break;
      case 2400:  bios0=5; break;
      case 4800:  bios0=6; break;
      case 9600:  bios0=7; break;
      case 19200: bios0=0; break;
      case 38400: bios0=1; break;
    }
    bios0 <<= 5;

    switch (byte_size)
    {
      case 5:
      case 6:
      case 7: bios0 |= 2; break;  // 7
      case 8: bios0 |= 3; break;  // 8
    }
    switch (parity)  // 0 none, 1=odd, 2=even
    {
      case 0:                 break; // none
      case 1: bios0 |= 0x8;   break; // odd
      case 2: bios0 |= 0x18;  break; // even
    }

    switch (stop_bits)  // 0=1, 1=1.5, 2=2
    {
      case 2: bios0 |= 0x4;  break; // 2
    }

    RegDX= port;
    RegAX= bios0 & 0xff;  // ah=func 0, al = config

    ASM {
      mov DX, RegDX
      mov AX, RegAX
      int 0x14
      mov RegAX, AX
    }
  }
  else
  {
    switch (baud)
    {
      case 110:   bval = 0;  break;
      case 150:   bval = 1;  break;
      case 300:   bval = 2;  break;
      case 600:   bval = 3;  break;
      case 1200:  bval = 4;  break;
      case 2400:  bval = 5;  break;
      case 4800:  bval = 6;  break;
      case 9600:  bval = 7;  break;
      case 19200: bval = 8;  break;
      case 38400: bval = 9;  break;
      case 57600: bval = 10; break;
      case 76800: bval = 11; break;
      case 115200:bval = 12; break;
      case 230400:bval = 13; break;
      case 460800:bval = 14; break;
      default  :  bval = 7;  break;
    }
    bios0 <<= 5;
   
    switch (byte_size)
    {
      case 5:
      case 6:
      case 7: bsize = 2; break;  // 7
      case 8: bsize = 3; break;  // 8
    }

    RegDX= port;
    RegCX= (bsize << 8) | bval;
    RegBX= (parity << 8) | stop_bits;
    RegAX= 0x400;  // ah=0x1e, al=0(break off)

    ASM {
      mov DX, RegDX
      mov CX, RegCX
      mov BX, RegBX
      mov AX, RegAX
      int 0x14
      mov RegAX, AX
    }
  }
  return(RegAX);
}

/*----------------------------------------------------------------
| FOSSgetcursor -
|-----------------------------------------------------------------*/
void FOSSgetcursor(int *x, int *y)
{
 WORD RegAX = 0x1200;
 WORD RegDX;

  ASM {
    mov AX, RegAX
    int 0x14
    mov RegDX, DX
  }
  *x = RegDX & 0xff;
  *y = RegDX >> 8;
}

/*----------------------------------------------------------------
| FOSSsetcursor -
|-----------------------------------------------------------------*/
void FOSSsetcursor(int x, int y)
{
 WORD RegAX = 0x1100;
 WORD RegDX = (y << 8) | x;

  ASM {
    mov AX, RegAX
    mov DX, RegDX
    int 0x14
  }
}

/*----------------------------------------------------------------
| FOSSscrchar -
|-----------------------------------------------------------------*/
void FOSSscrchar(char c)
{
 WORD RegAX = 0x1500 | c;

  ASM {
    mov ax, RegAX
    int 0x14
  }
}

/*----------------------------------------------------------------
| FOSSloopback -
|-----------------------------------------------------------------*/
void FOSSloopback(int port, int setflag)
{
 WORD mcr;
 WORD RegDX = port;
 WORD RegAX = 0x1f00; // func 1F, sub 0 , read MCR
 WORD RegBX;

  ASM {
    mov DX, RegDX
    mov AX, RegAX
    int 0x14
    mov RegBX, BX
  }
  mcr = RegBX;
  if (setflag)
       mcr |=  0x0010;  // loopback bit
  else mcr &= ~0x0010;

  RegBX = mcr;
  RegDX= port;
  RegAX= 0x1f01;  // func 1F, sub 0 , write MCR
  ASM {
    mov DX, RegDX
    mov BX, RegBX
    mov AX, RegAX
    int 0x14
  }
}

/*----------------------------------------------------------------
| FOSSflushtx -
|-----------------------------------------------------------------*/
void FOSSflushtx(int port)
{
 WORD RegDX= port;
 WORD RegAX= 0x0900;  // func 9, flush tx

  ASM {
    mov DX, RegDX
    mov AX, RegAX
    int 0x14
  }
}

/*----------------------------------------------------------------
| FOSSflushrx -
|-----------------------------------------------------------------*/
void FOSSflushrx(int port)
{
 WORD RegDX= port;
 WORD RegAX= 0x0a00;  // func a, flush rx

  ASM {
    mov DX, RegDX
    mov AX, RegAX
    int 0x14
  }
}

/*----------------------------------------------------------------
| FOSSbreak -
|-----------------------------------------------------------------*/
void FOSSbreak(int port, int setflag)
{
 WORD RegDX= port;
 WORD RegAX= 0x1a00;  // func a, flush rx

 if (setflag)
   RegAX |= 1;

  ASM {
    mov DX, RegDX
    mov AX, RegAX
    int 0x14
  }
}

/*----------------------------------------------------------------
| FOSSdtr -
|-----------------------------------------------------------------*/
void FOSSdtr(int port, int setflag)
{
 WORD RegDX= port;
 WORD RegAX= 0x0600;  // func 6, set/clear dtr

 if (setflag)
   RegAX |= 1;

  ASM {
    mov DX, RegDX
    mov AX, RegAX
    int 0x14
  }
}

/*----------------------------------------------------------------
| FOSSrts -
|-----------------------------------------------------------------*/
void FOSSrts(int port, int setflag)
{
 WORD mcr;
 WORD RegDX= port;
 WORD RegAX= 0x1f00;  // func 1F, sub 0 , read MCR
 WORD RegBX;

  ASM {
    mov DX, RegDX
    mov AX, RegAX
    int 0x14
    mov mcr, BX
  }

  if (setflag)
       mcr |=  0x0002;
  else mcr &= ~0x0002;

  RegBX = mcr;
  RegDX= port;
  RegAX= 0x1f01;  // func 1F, sub 0 , write MCR

  ASM {
    mov DX, RegDX
    mov BX, RegBX
    mov AX, RegAX
    int 0x14
  }
}

/*----------------------------------------------------------------
| FOSSgetMCR - Get the modem control register.
 BL = Modem control register, where:
      Bits 7-5 = 0 (Reserved)
      Bit 4    = 1 Loopback mode
      Bit 3    = 1 OUT2 (interrupts) enabled
      Bit 2    = 1 OUT1 active
      Bit 1    = 1 Request to send active
      Bit 0    = 1 Data terminal ready (DTR) active
|-----------------------------------------------------------------*/
BYTE FOSSgetMCR(int port)
{
 WORD mcr;
 WORD RegDX= port;
 WORD RegAX= 0x1f00;  // func 1F, sub 0 , read MCR
 WORD RegBX;

  ASM {
    mov DX, RegDX
    mov AX, RegAX
    int 0x14
    mov mcr, BX
  }
  return (BYTE) mcr;
}

/*----------------------------------------------------------------
| FOSSsetMCR - Set the modem control register.
|-----------------------------------------------------------------*/
BYTE FOSSsetMCR(int port, BYTE new_mcr)
{
 WORD mcr;
 WORD RegDX= port;
 WORD RegAX= 0x1f01;  // func 1F, sub 1 , write MCR
 WORD RegBX;

  RegBX = new_mcr;

  ASM {
    mov DX, RegDX
    mov BX, RegBX
    mov AX, RegAX
    int 0x14
    mov mcr, BX
  }
  return (BYTE) mcr;
}

/*----------------------------------------------------------------
| FOSSinit -
|-----------------------------------------------------------------*/
unsigned int FOSSinit(int port)
{
 WORD RegDX= port;
 WORD RegAX= 0x0400;  // init

  ASM {
    mov DX, RegDX
    mov AX, RegAX
    int 0x14
    mov RegAX, AX
  }
  return(RegAX);
}

/*----------------------------------------------------------------
| FOSSclose -
|-----------------------------------------------------------------*/
void FOSSclose(int port)
{
 WORD RegDX= port;
 WORD RegAX= 0x0500;  // close

  ASM {
    mov DX, RegDX
    mov AX, RegAX
    int 0x14
  }
}

/*----------------------------------------------------------------
| FOSSbaud - Bios 00 function
|-----------------------------------------------------------------*/
unsigned int FOSSbaud(int port, unsigned char baud)
{
 WORD RegDX= port;
 WORD RegAX= (WORD) baud;  // ???

  ASM {
    mov DX, RegDX
    mov AX, RegAX
    int 0x14
    mov RegAX, AX
  }
  return(RegAX);
}

/*----------------------------------------------------------------
| FOSSbaudExt - Extended Bios 04 function
|-----------------------------------------------------------------*/
unsigned int FOSSbaudExt(int port, WORD baud, WORD byte_size,
                                WORD parity, WORD stop_bits)
{
 WORD RegDX;
 WORD RegCX;
 WORD RegBX;
 WORD RegAX;

  RegDX= port;
  RegCX= (byte_size << 8) | baud;
  RegBX= (parity << 8) | stop_bits;
  RegAX= 0x1e00;  // ah=0x1e, al=0(break off)

  ASM {
    mov DX, RegDX
    mov CX, RegCX
    mov BX, RegBX
    mov AX, RegAX
    int 0x14
    mov RegAX, AX
  }

  // last call disables access to Bios00 baud rate for some reason,
  // this following call re-enables ability to do Bios00 baud rate setup.
  ASM {
    mov CX, 0ffH
    mov AX, 1e00H
    int 0x14
  }
  return(RegAX);
}

/*----------------------------------------------------------------
| FOSSputc -
|-----------------------------------------------------------------*/
void FOSSputc(int port, unsigned char c)
{
 WORD RegDX= port;
 WORD RegAX = c | 0x100;

  ASM {
   mov DX, RegDX
   mov AX, RegAX
   int 0x14
  }
}

/*----------------------------------------------------------------
| FOSSputc_0B - No wait variety.
|-----------------------------------------------------------------*/
void FOSSputc_0B(int port, unsigned char c)
{
 WORD RegDX= port;
 WORD RegAX = c | 0xB00;

  ASM {
   mov DX, RegDX
   mov AX, RegAX
   int 0x14
  }
}

/*----------------------------------------------------------------
| FOSSgetc -
|-----------------------------------------------------------------*/
int FOSSgetc(int port)
{
 WORD RegDX= port;
 WORD RegAX = 0x200;

  ASM {
   mov DX, RegDX
   mov AX, RegAX
   int 0x14
   mov RegAX, AX
  }
  return(RegAX);
}

/*----------------------------------------------------------------
| FOSSstatus -
  Basic Bios-03 service:
  see LNSTAT_XXXX and MDMSTAT_XXXX definitions.  Only difference is that
  fossil has MDMSTAT_DCD always set.
|-----------------------------------------------------------------*/
unsigned int FOSSstatus(int port)
{
 WORD RegDX= port;
 WORD RegAX = 0x300;

  ASM {
   mov DX, RegDX
   mov AX, RegAX
   int 0x14
   mov RegAX, AX
  }
  return(RegAX);
}

/*----------------------------------------------------------------
| FOSSflowCtrl -
|-----------------------------------------------------------------*/
void FOSSflowCtrl(int port, unsigned char mode)
{
 WORD RegDX= port;
 WORD RegAX = 0xf00 | mode;

  ASM {
   mov DX, RegDX
   mov AX, RegAX
   int 0x14
  }
}

/*----------------------------------------------------------------
| FOSSpeek -
|-----------------------------------------------------------------*/
unsigned int FOSSpeek(int port)
{
 WORD RegDX= port;
 WORD RegAX = 0xc00;

  ASM {
   mov DX, RegDX
   mov AX, RegAX
   int 0x14
   mov RegAX, AX
  }
  return(RegAX);
}

/*----------------------------------------------------------------
| FOSSctlck -
|-----------------------------------------------------------------*/
unsigned int FOSSctlck(int port, unsigned char flag)
{
 WORD RegDX= port;
 WORD RegAX = 0x1000 | flag;

  ASM {
   mov DX, RegDX
   mov AX, RegAX
   int 0x14
   mov RegAX, AX
  }
  return(RegAX);
}

/*----------------------------------------------------------------
| FOSSreadblk -
|-----------------------------------------------------------------*/
unsigned int FOSSreadblk(int port, int count, unsigned char *buffer)
{
 WORD RegDX= port;
 WORD RegCX = count;
 WORD RegES = FP_SEG(buffer);
 WORD RegDI = FP_OFF(buffer);
 WORD RegAX = 0x1800;

  ASM {
   mov DI, RegDI
   mov ES, RegES
   mov DX, RegDX
   mov CX, RegCX
   mov AX, RegAX
   int 0x14
   mov RegAX, AX
  }
  return(RegAX);
}

/*----------------------------------------------------------------
| FOSSwriteblk -
|-----------------------------------------------------------------*/
unsigned int FOSSwriteblk(int port, int count, unsigned char *buffer)
{
 WORD RegDX= port;
 WORD RegAX = 0x1900;
 WORD RegCX = count;
 WORD RegES = FP_SEG(buffer);
 WORD RegDI = FP_OFF(buffer);

  ASM {
   mov DI, RegDI
   mov ES, RegES
   mov DX, RegDX
   mov CX, RegCX
   mov AX, RegAX
   int 0x14
   mov RegAX, AX
  }
  return(RegAX);
}

/*----------------------------------------------------------------
| FOSSset_hook - Custom call to chain in single application to
   interrupt time code(18ticks per sec timer or 274Hz IRQ poll.)
   proc_addr - ptr to far call void hook_func(void);
   Can be used to hook timer/irq and do things like double buffering,
   simple io-tasks, etc.
|-----------------------------------------------------------------*/
unsigned int FOSSset_hook(void far *proc_addr)
{
 WORD RegAX = 0x1600;
 WORD RegES = FP_SEG(proc_addr);
 WORD RegDX = FP_OFF(proc_addr);

  ASM {
   mov ES, RegES
   mov DX, RegDX
   mov AX, RegAX
   int 0x14
   mov RegAX, AX
  }

  return  RegAX;  // 0=ok
}

/*----------------------------------------------------------------
| FOSSgetinfo -
|-----------------------------------------------------------------*/
unsigned int FOSSgetinfo(int port, FossInfo *fi)
{
 WORD RegDX= port;
 WORD RegAX = 0x1b00;
 WORD RegCX = sizeof(FossInfo);
 WORD RegES = FP_SEG(fi);
 WORD RegDI = FP_OFF(fi);


  ASM {
   mov DI, RegDI
   mov ES, RegES
   mov DX, RegDX
   mov CX, RegCX
   mov AX, RegAX
   int 0x14
   mov RegAX, AX
  }

  if (RegAX > 0)  // num bytes xfered
    return 0; // ok
  return 1; // err
}


